%% clear
clc;
clear;
close all;
format compact;
format short g;

%% Set up FEM struct and pre-assemble
fem = getInitFem;
prob.fem.dof = getDof(fem);

prob.theta(1).fun = @(mu) mu(1);
prob.theta(2).fun = @(mu) mu(2);

prob = performPreAssembling(fem,prob);

%% Compute matrices for discrete inner porduct
[fem.const{2:2:end}] = deal(1);
[C,D] = assemble(fem,'Out',{'C','D'});

L2 = D;
L2 = (L2+L2')/2;

H1 = C+D;
H1 = (H1+H1')/2;

En = C;
En = (En+En')/2;

prob.fem.innProd.L2 = L2;
prob.fem.innProd.H1 = H1;
prob.fem.innProd.En = En;

POD_Norm = 'H1';
RB_Norm = 'H1';
mu_bar = [1;1];

prob.fem.innProd.refparam = mu_bar;
prob.fem.innProd.refalpha = getCoercivity(prob, mu_bar, RB_Norm)
prob.fem.innProd.refthetas =  zeros(length(prob.fem.pre.lhs),1);
for q=1:length(prob.fem.pre.lhs)
  prob.fem.innProd.refthetas(q) = prob.fem.pre.lhs(q).theta(mu_bar);
end

%% RB training using Greedy

% parameter space w.r.t. \mu_1
D = [.01 100];
logD = log10(D);

% number of snapshots n_train
n_train = 101;

% parameter training set
Xi_train = [logspace(logD(1),logD(2),n_train); ...
            ones(1,n_train)];

% initial parameter
S_N0 = [min(D); 1];

%Greedy training
prob.RB = trainRB_Greedy(prob, Xi_train, RB_Norm);

figure
semilogy(prob.RB.Greedy_errors, 'b-x');
title('Greedy Error during training (primal problem)')
xlabel('N');
ylabel('$\max\limits_{\mu \in \Xi_{\mathrm{train}}} \Delta_N (\mu)$', 'Interpreter','LaTex')

%% RB tests for simple output approximation

n_test = 40;
Xi_test = [10.^(sort(rand(1,n_test)*diff(logD)+logD(1))); ...
           ones(1,n_test)];

% compute RB errors and error estimates
err_out_RB = zeros(n_test,prob.RB.Nmax);
err_out_est = zeros(n_test,prob.RB.Nmax);


% ======= TBD ==========

for n=1:n_test
  mu = Xi_test(:,n);
  
  u = getSol(prob, mu);
  % Calculate true output:
  % s =  

  for N = 1:prob.RB.Nmax
    
    % Calculate (simple) output approximation
    % s_RB = ...
    
    % Calculate true error and (simple) error bound
    %err_out_RB(n,N) = ...
    %err_out_est(n,N) = ...
  end
  
end

% ======================

figure;
loglog(Xi_test(1,:),err_out_RB,'-x');
hold on
loglog(Xi_test(1,:), err_out_est, '--o');
legend(num2str((1:prob.RB.Nmax)','N=%d'));
title('Output Error (-x) and simple error bound (--o) over test set');
xlabel('\mu')
ylabel('$\vert s^{\mathcal N}(\mu) - s_N(\mu)\vert$ bzw. $\Vert \ell \Vert_{X\prime}\cdot\Delta_N(\mu)$', 'Interpreter', 'LaTex')
hold off

[max_err_out_RB, ind] = max(err_out_RB,[],1);
figure
semilogy(max_err_out_RB, '-x');
hold on
semilogy(diag(err_out_est(ind, :)), '--o');
title('Maximum Output Error and corresponding error bound over test set');
xlabel('N')
ylabel('$\max\limits_{\mu \in \Xi_{\mathrm{test}}} \vert s^{\mathcal N}(\mu) - s_N(\mu)\vert$ bzw. $\Delta_N^{out} (\mu)$', 'Interpreter', 'LaTex')
legend('Maximum Error', 'Error Bound')
hold off

%% RB training for Dual problem

%Greedy training
prob.RBDual = trainDualRB_Greedy(prob, Xi_train, RB_Norm);

figure
semilogy(prob.RBDual.Greedy_errors, 'b-x');
title('Greedy Error during training (dual problem)')
xlabel('N');
ylabel('$\max\limits_{\mu \in \Xi_{\mathrm{train}}} \Delta_N (\mu)$', 'Interpreter','LaTex')

%% RB tests for primal-dual output approximation

% compute RB errors and error estimates
err_out_RB_pd = zeros(n_test,min(prob.RB.Nmax, prob.RBDual.Nmax));
err_out_est_pd = zeros(n_test,min(prob.RB.Nmax, prob.RBDual.Nmax));

innProd = prob.fem.innProd.(prob.RB.Norm);

% ======= TBD ==========

for n=1:n_test
  mu = Xi_test(:,n);
  
  u = getSol(prob, mu);
  % Calculate true output:
  % s =  
  
   for N = 1:min(prob.RB.Nmax, prob.RBDual.Nmax)
                
    % Calculate primal-dual output approximation
    % using N = Ntilde
    % s_RB = 
        
    % Calculate true error and primal-dual error bound
    %err_out_RB_pd(n,N) = ...
    %err_out_est_pd(n,N) = ...
    
    end
  
end

% ======================


figure;
loglog(Xi_test(1,:),err_out_RB_pd,'-x');
hold on
loglog(Xi_test(1,:), err_out_est_pd, '--o');
legend(num2str((1:min(prob.RB.Nmax, prob.RBDual.Nmax))','N=Ntilde=%d'));
title('Output Error (-x) and Primal-Dual error bound (--o) over test set');
xlabel('\mu')
ylabel('$\vert s^{\mathcal N}(\mu) - s_N(\mu)\vert$ bzw. $\Vert \ell \Vert_{X\prime}\cdot\Delta_N(\mu)$', 'Interpreter', 'LaTex')
hold off

[max_err_out_RB_pd, ind] = max(err_out_RB_pd,[],1);
figure
semilogy(max_err_out_RB_pd, '-x');
hold on
semilogy(diag(err_out_est_pd(ind, :)), '--o');
title('Maximum Output Error and corresponding primal-dual error bound over test set');
xlabel('N')
ylabel('$\max\limits_{\mu \in \Xi_{\mathrm{test}}} \vert s^{\mathcal N}(\mu) - s_N(\mu)\vert$ bzw. $\Delta_{N,\widetilde N}^{out} (\mu)$', 'Interpreter', 'LaTex')
legend('Maximum Error', 'Error Bound')
hold off

%% Plot effectivities

eff =  err_out_est ./ err_out_RB;
eff_pd =  err_out_est_pd ./ err_out_RB_pd;
figure
subplot(1,3,1);
semilogx(Xi_test(1,:), eff(:,1), 'b-x')
hold
semilogx(Xi_test(1,:), eff_pd(:,1), 'r-o')
legend('Simple Bound', 'Primal-Dual Bound');
title('N=1');
xlabel('\mu')
ylabel('Effectivity')
hold off

subplot(1,3,2);
semilogx(Xi_test(1,:), eff(:,2), 'b-x')
hold
semilogx(Xi_test(1,:), eff_pd(:,2), 'r-o')
legend('Simple Bound', 'Primal-Dual Bound');
title('N=2');
xlabel('\mu')
ylabel('Effectivity')
hold off

subplot(1,3,3);
semilogx(Xi_test(1,:), eff(:,3), 'b-x')
hold
semilogx(Xi_test(1,:), eff_pd(:,3), 'r-o')
legend('Simple Bound', 'Primal-Dual Bound');
title('N=3');
xlabel('\mu')
ylabel('Effectivity')
hold off

