function prob = performPreAssembling(fem,prob)

  prob.fem.pre.lhs = struct('theta',{},'K',{});
  
  %% Pre-Assembly for primal problem
  for q=1:numel(prob.theta)

    theta = zeros(numel(prob.theta),1);
    theta(q) = 1;
    fem.const(2:2:end) = num2cell(theta);
    
    [K1,L1] = assemble(fem,'Out',{'K','L'});
    
    theta = 2*theta;
    fem.const(2:2:end) = num2cell(theta);
    
    [K2,L2] = assemble(fem,'Out',{'K','L'});

    Kconst = 2*K1-K2;
    Klin = K1-Kconst;
    Lconst = 2*L1-L2;
    Llin = L1-Lconst;

    if (q==1)
      if (nnz(Kconst)>0)
        prob.fem.pre.lhs(1) = struct('theta',@(mu) 1,'K',Kconst);
      end
      if any(Lconst)
        prob.fem.pre.rhs.theta = @(mu) 1;
        prob.fem.pre.rhs.L = Lconst;
      else
        prob.fem.pre.rhs.theta = @(mu) [];
        prob.fem.pre.rhs.L = [];
      end
    end
      
    if (nnz(Klin)>0)
      prob.fem.pre.lhs(end+1) = struct('theta',prob.theta(q).fun,'K',Klin);
    end
  
    if any(Llin)
      prob.fem.pre.rhs.theta = @(mu) [prob.fem.pre.rhs.theta(mu); ...
                                      prob.theta(q).fun(mu)];
      prob.fem.pre.rhs.L = [prob.fem.pre.rhs.L Llin];
    end
  end
  
  %% Output Pre-Assembly
  fem_out = initFemOutput(fem);
  
  F = assemble(fem_out, 'Out', 'F');
  
  % Test
  fem_out.sol = femsol(F);
  meshplot(fem_out)
  hold
  postplot(fem_out, 'tridata', 'u');
  hold off
  
  prob.fem.pre.out.L = F;
  prob.fem.pre.out.theta = @(mu) 1;