function RB = trainRB_Greedy(prob,Xi_train,Norm,Nmax,tol)

%% Initialize parameters and storage

  Nmax = eval('Nmax','100');
  tol = eval('tol','5e-6');

% store Xi_train  
  RB.Xi_train = Xi_train;
% store norm to be used  
  RB.Norm = Norm;
  if isfield(prob.fem.innProd, 'refparam')
      RB.innProd.refparam = prob.fem.innProd.refparam;
  end
  if isfield(prob.fem.innProd, 'refalpha')
      RB.innProd.refalpha = prob.fem.innProd.refalpha;
  end
  if isfield(prob.fem.innProd, 'refthetas')
      RB.innProd.refthetas = prob.fem.innProd.refthetas;
  end
  
% store theta function for both sides and initialize RB matrices
  RB.pre.rhs.theta = prob.fem.pre.rhs.theta;
  RB.pre.lhs = struct('theta',{}, 'K',{});
  RB.pre.rhs.L = [];
  for q=1:length(prob.fem.pre.lhs)
    RB.pre.lhs(q).theta = prob.fem.pre.lhs(q).theta;
  end
  
  
% initialize S, Basis and Riesz Representors
  RB.S = zeros(size(Xi_train,1),0);
  RB.Basis = zeros(prob.fem.dof.num,0);
  RB.RieszReprs = zeros(length(prob.fem.dof.ind),0);
  RB.innProdRieszReprs = [];
  RB.Greedy_errors = [];
  RB.Nmax = 0;

  
  % compute Riesz representors 
  innProd = prob.fem.innProd.(RB.Norm);
  innProd = innProd(prob.fem.dof.ind,prob.fem.dof.ind);
% ... for rhs 
    for q=1:size(RB.pre.rhs.theta,2)
      RB.RieszReprs(:,end+1) = ...
        innProd\prob.fem.pre.rhs.L(prob.fem.dof.ind,q);
    end
    RB.innProdRieszReprs = RB.RieszReprs'*innProd*RB.RieszReprs;

  
%% Training
  
% start greedy
  while (size(RB.Basis,2)<Nmax)

  % compute error estimator for all remaining values in Xi_train
    errEst = ones(size(Xi_train,2),1);

    for i=1:length(errEst)
      mu = Xi_train(:,i);
    % compute RB approximation
      u_apprCoeffs = getRBAppr(RB,mu);
    % compute error estimator  
      errEst(i) = getErrEst(RB,mu,u_apprCoeffs);
    end

  % determine index of argmax in Xi_train
    [errEst_max,errEst_max_ind] = max(errEst);
    
  % store Greedy errors
  RB.Greedy_errors(end+1) = errEst_max;

  % break, if maximum error estimator is less than tol
    if (errEst_max<tol)
      break;
    end
    
  % add solution for identified parameter to basis
    RB = addBasisFun(prob,RB,Xi_train(:,errEst_max_ind));

  % remove identified parameter from Xi_train  
    Xi_train(:,errEst_max_ind) = [];
    
  end
  
% store Nmax
  RB.Nmax = size(RB.Basis,2);

end  
  
%% add orthonormalized u(mu) to RB basis
function RB = addBasisFun(prob,RB,mu)

% compute solution
  u = getSol(prob,mu);

% orthonormalize 
  if (size(RB.S,2)==0)
    r = u;
  else
    r = u - RB.Basis*(RB.Basis'*prob.fem.innProd.(RB.Norm)*u);
  end
  RB.Basis(:,end+1) = r/sqrt(r'*prob.fem.innProd.(RB.Norm)*r);

% compute new pre-assembled RB quantities
  % lhs
  for q=1:size(prob.fem.pre.lhs,2)
    RB.pre.lhs(q).K = zeros(size(RB.Basis,2));
    K = RB.Basis'*prob.fem.pre.lhs(q).K*RB.Basis;
    RB.pre.lhs(q).K = K;
  end
  % rhs
  RB.pre.rhs.L = RB.Basis'*prob.fem.pre.rhs.L;

% compute Riesz representors
  innProd = prob.fem.innProd.(RB.Norm);
  innProd = innProd(prob.fem.dof.ind,prob.fem.dof.ind);
% ... for lhs
  for q=1:size(RB.pre.lhs,2)
    Kn = -prob.fem.pre.lhs(q).K*RB.Basis(:,end);
    RB.RieszReprs(:,end+1) = innProd\Kn(prob.fem.dof.ind);
  end
% compute inner product of duals
  RB.innProdRieszReprs = RB.RieszReprs'*innProd*RB.RieszReprs;
  
% add mu to S
  RB.S = [RB.S mu];  
end  